--local inspect = require ("inspect")

local vehicle = require ("vehicle")
local window = require ("zoom-and-move-window")
rays = require("rays")

local track = require ("track")
local grid = require ("grid")
right_line = track[1]
left_line = track[2]
points = track[3]

local fontName = "8bitOperatorPlus-Regular.ttf"
local Font = love.graphics.newFont( fontName, 11 )
Font:setFilter( "nearest", "nearest" )
love.graphics.setFont(Font)



function love.load()
	love.math.setRandomSeed(love.timer.getTime())

	window:load()
	window.translate.x = love.graphics.getWidth( )/2
	window.translate.y = love.graphics.getHeight( )/2
	
	cars = {}
	for i = 1, 2 do
		local car = vehicle:new(i)
		local base_angles = {180, 0, 15, -15, 30, -30, 45, -45, 90, -90}
--		for j = 1, car.rays.amount do
--			local base_angle = (j-1)*2*math.pi/car.rays.amount
		for j, base_angle in pairs (base_angles) do
			base_angle = math.rad(base_angle)
			local ray = rays:new({
					name = i..'-'..j, 
					color = car.color, 
					base_angle = base_angle})
--			print (inspect(ray))
			car.rays[j] = ray
		end
		cars[i] = car
		
	end
	
--	print(cars[1].name)
--	print(cars[2].name)
end


function love.update(dt)
	window:update(dt)
	
	local car = cars[1]
	car:update(dt)
	local shortest_ray = 800
	for i, ray in ipairs (car.rays) do
--		print ('update ' .. inspect(ray))
		ray.angle = car.directon + ray.base_angle
		ray.position = car.position
		ray:update(dt, track)
		if ray.length and (ray.length < shortest_ray) then
			shortest_ray = ray.length
		end
	end
	if shortest_ray <=10 then
		car.color = {1,0,0}
	end
--	cars[2]:update(dt)
end


function love.draw()
	window:draw()
	
	
	
	
	
	
	-- grid
	love.graphics.setColor(0.1,0.1,0.1)
	love.graphics.setLineWidth( 0.5 )
	love.graphics.lines(grid)
	
	-- start point
	love.graphics.setColor(0.75,0.75,0.75)
	love.graphics.line(-1000, 0, 1000, 0)
	love.graphics.line(0, -1000, 0, 1000)
	
	
	-- track
	love.graphics.setColor(0,1,0)
	love.graphics.line(right_line)
	love.graphics.setColor(1,1,0)
	love.graphics.line(left_line)
	
	-- track connections

	
	love.graphics.setColor(1,1,1)
--	for i = 1, (#points-2), 2 do
--		local size = i%20
--		love.graphics.circle("line", points[i], points[i+1], size)
--	end
	
	for _, line in pairs ({right_line, left_line}) do
		for i = 1, (#line-2), 2 do
			local x = line[i]
			local y = line[i+1]
			love.graphics.circle("line", x,y, 5)
			if (window.zoom >= 2) then
				love.graphics.print(' x:'..x..' y:'..y..' i:'..(i+1)/2, x, y, -0.1)
			end
		end
	end
	
	
	local car = cars[1]
	car:draw()
	for j, ray in ipairs (car.rays) do
		ray:draw((window.zoom >= 2))
	end
	
	
--	cars[2]:draw()
	
	
	
	-- debug GUI
	local mx = love.mouse.getX()
	local my = love.mouse.getY()
	love.graphics.origin()
	love.graphics.setColor(0,1,0)
	
	local y = 32
	love.graphics.print ('Debug GUI:', 1, y)
	y = y + Font:getHeight( )
	love.graphics.print('x: ' .. mx ..', y: '.. my, 1, y)
	y = y + Font:getHeight( )
	love.graphics.print('x: ' .. (mx-window.translate.x)/window.zoom ..
		', y: '.. (my-window.translate.y)/window.zoom, 1, y) 
	y = y + Font:getHeight( )
	love.graphics.print('zoom ' .. window.zoom, 1, y)
	y = y + Font:getHeight( )
	love.graphics.print('tr ' .. window.translate.x ..', '.. window.translate.y, 1, y) 
	y = y + Font:getHeight( )
	love.graphics.print('dscale: ' .. window.dscale , 1, y)
	y = y + Font:getHeight( )
	love.graphics.print("Current FPS: "..tostring(love.timer.getFPS( )), 10, y)
end


function love.mousepressed(x, y, k)
	window:mousepressed(x, y, k)
end

 
function love.wheelmoved(x, y)
	window:wheelmoved(x, y)
end

function love.keypressed(k)
	if k == 'escape' then
		love.event.quit()
--	elseif k == 'a' then
--		size = size and size+1 or 1
--		local myFont = love.graphics.newFont( fontName, size )
--		myFont:setFilter( "nearest", "nearest" )
--		love.graphics.setFont( myFont )
--		print('size: '..size)
	end
end








