local vehicle = {} -- ObjectClass
local vehicleMT = {__index = vehicle} -- ObjectClassMT


function vehicle:new(name)
--	self.__index = self
	
	car = {}
--	car.update = vehicle.update
--	car.draw = vehicle.draw
--	car.drawWheels = vehicle.drawWheels
	car.name = name
	car.position = {x=0,y=0}
	car.directon = 3*math.pi/4
--	car.directon = 2*math.pi/4
--	car.directon = 0
	car.curvature = 0
	car.delta_curvature = 2
	car.max_curvature = 1
	car.speed = 0
	car.max_speed = 40
	car.friction = 2
	car.max_delta_force = 10
	car.mass = 10
	car.length = 30
	car.width = 20
	car.shape = {0, car.width/2, car.length, car.width/2-2, car.length, -car.width/2+2, 0, -car.width/2}
	car.wheel = {x=car.length, y=10, radius=6, c_left=0, c_right=0, color = {1, 1, 0.5}} -- position, radius, angle (yellow)
	car.color = {0.5+0.5*math.random(),0.5+0.5*math.random(),0.5+0.5*math.random()}
	car.rays = {amount = 8}
	return setmetatable(car, vehicleMT)
end


function vehicle:update(dt)
	
	-- update speed and curvature

	if love.keyboard.isDown("w") then
		if self.speed > 0 then
			self.speed = self.speed + dt*self.max_delta_force*((self.max_speed - self.speed)/self.max_speed)
		else -- brake
			self.speed = self.speed + 2*dt*self.max_delta_force
		end
		if self.speed > self.max_speed then self.speed = self.max_speed end
	elseif love.keyboard.isDown("s") then
		if self.speed < 0 then
			self.speed = self.speed - 0.5*dt*self.max_delta_force*((self.max_speed - self.speed)/self.max_speed)
		else -- brake
			self.speed = self.speed - 2*dt*self.max_delta_force
		end
		if self.speed < -self.max_speed then self.speed = -self.max_speed end
	else -- friction
		if self.speed > 0 then
			self.speed = math.max (self.speed - dt*self.friction, 0)
		elseif self.speed < 0 then
			self.speed = math.min (self.speed + dt*self.friction, 0)
		end
	end
	if love.keyboard.isDown("d") then
		self.curvature = math.min(self.curvature + dt*self.delta_curvature,  self.max_curvature*((self.max_speed - self.speed)/self.max_speed))
	elseif love.keyboard.isDown("a") then
		self.curvature = math.max(self.curvature - dt*self.delta_curvature, -self.max_curvature*((self.max_speed - self.speed)/self.max_speed))
	else -- friction
		self.curvature = 0.9*self.curvature
	end
	
	-- update position and rotation
	self.position.x = self.position.x + self.speed * math.cos(self.directon)
	self.position.y = self.position.y + self.speed * math.sin(self.directon)
	self.directon = self.directon + dt*self.curvature*self.speed * ((1+self.max_speed - self.speed)/self.max_speed)
--	self.directon = self.directon + dt*self.curvature*self.speed
end


function vehicle:drawWheels()
	local x = self.wheel.x
	local y = self.wheel.y -- self.position.y
	local c = self.curvature/2
	
	if middle_wheel then -- disabled
		local dx = self.wheel.radius*math.cos(c)
		local dy = self.wheel.radius*math.sin(c)
		local wheel = {x+dx, dy, x-dx, -dy}
		love.graphics.setColor({1, 1, 1})
		love.graphics.line (change_position(rotate_table(wheel,  self.directon), self.position))
	end
	
	local dx_left = self.wheel.radius*math.cos(c)
	local dy_left = self.wheel.radius*math.sin(c)
	local wheel_left = {x+dx_left, y+dy_left, x-dx_left, y-dy_left} -- yellow
	love.graphics.setColor({1, 1, 0.5})
	love.graphics.line (change_position(rotate_table(wheel_left,  self.directon), self.position))
	
	
	local dx_right = self.wheel.radius*math.cos(c)
	local dy_right = self.wheel.radius*math.sin(c)
	local wheel_right = {x+dx_right, -y+dy_right,  x-dx_right, -y-dy_right} -- green
	love.graphics.setColor({0.5, 1, 0.5})
	love.graphics.line (change_position(rotate_table(wheel_right, self.directon), self.position))
end


function rotate_table (tabl, angle)
	local new_table = {}
	for i = 1, (#tabl-1), 2 do
		local x = tabl[i]
		local y = tabl[i+1]
		local x1 = x * math.cos(angle) - y * math.sin(angle)
		local y1 = x * math.sin(angle) + y * math.cos(angle)
		new_table[i] = 	 x1
		new_table[i+1] = y1
	end
	return new_table
end


function change_position (tabl, position)
	local new_table = {}
	local x = position.x or position[1]
	local y = position.y or position[2]
	for i = 1, (#tabl-1), 2 do
		new_table[i] = 	 tabl[i]   + x
		new_table[i+1] = tabl[i+1] + y
	end
	return new_table
end

function vehicle:draw()
	love.graphics.setColor(self.color)
	love.graphics.circle("line", self.position.x, self.position.y, self.width/2)
	love.graphics.line (change_position(rotate_table(self.shape, self.directon), self.position))
	
	self:drawWheels()
end


return vehicle